@@ -391,6 +391,10 @@ namespace DisplayPort
             return maxModeBwRequired;
         }
 
+        bool getStuffDummySymbolsFor128b132b() const { return processedEdid.WARData.bStuffDummySymbolsFor128b132b; }
+        bool getStuffDummySymbolsFor8b10b() const { return processedEdid.WARData.bStuffDummySymbolsFor8b10b; }
+        bool getApplyStuffDummySymbolsWAR() const { return processedEdid.WARFlags.bApplyStuffDummySymbolsWAR; }
+
         virtual void queryGUID2();
 
         virtual bool getSDPExtnForColorimetrySupported();
@@ -433,6 +437,8 @@ namespace DisplayPort
         virtual bool    isMarkedForDeletion() {return bisMarkedForDeletion;};
         virtual bool    getRawDscCaps(NvU8 *buffer, NvU32 bufferSize);
         virtual bool    setRawDscCaps(const NvU8 *buffer, NvU32 bufferSize);
+        virtual bool    setValidatedRawDscCaps(NvU8 *buffer, NvU32 bufferSize);
+        virtual bool    validatePPSData(DSCPPSDATA *pPps);
 
         virtual AuxBus::status dscCrcControl(NvBool bEnable, gpuDscCrc *dataGpu, sinkDscCrc *dataSink);
 
@@ -508,6 +514,9 @@ namespace DisplayPort
         virtual bool getDeviceSpecificData(NvU8 *oui, NvU8 *deviceIdString,
                                            NvU8 *hwRevision, NvU8 *swMajorRevision,
                                            NvU8 *swMinorRevision);
+        virtual bool getParentSpecificData(NvU8 *oui, NvU8 *deviceIdString,
+                                           NvU8 *hwRevision, NvU8 *swMajorRevision,
+                                           NvU8 *swMinorRevision);
 
         virtual bool setModeList(DisplayPort::DpModesetParams *pModeList, unsigned numModes);
     };
@@ -529,6 +538,7 @@ namespace DisplayPort
             bool                     retryRemoteBKSVReadMessage;
             bool                     retryRemoteBCapsReadMessage;
             bool                     retryRemote22BCapsReadMessage;
+            bool                     tryRemote1XCaps;
             bool                     bBKSVReadMessagePending;
             bool                     bBCapsReadMessagePending;
 
@@ -554,6 +564,7 @@ namespace DisplayPort
 
             bool hdcpValidateKsv(const NvU8 *ksv, NvU32 Size);
             void handleRemoteDpcdReadDownReply(MessageManager::Message * from);
+            void readRemoteHdcp1xCaps(void);
             void messageFailed(MessageManager::Message * from, NakData * nakData);
             void messageCompleted(MessageManager::Message * from);
     };
